#include "stdafx.h"

const int DRAGBLOCKSIZE = 8;
const char * g_pchMovieFilter = "Movie file (*.gmv)\0*.gmv\0All Files (*.*)\0*.*\0\0";
const char * g_pchMovieExtension = "gmv";
enum CAPMODE { NOCAP, DATAMOVE, DATASIZE, DATAFRAME, STATEMOVE, STATESIZE, STATEFRAME } ;

//-------------------------------------------------------------------------
class CVwMovie : public CDocumentDialog { 
	public :
		CMovie m_oMovie;
		int m_iDataPos;				// offset from left edge to current pos in data window 

		//-- mouse drag support for data move / size 
		CAPMODE m_iCapture;
		int m_iXHit, m_iYHit;
		int m_iXDrag, m_iYDrag;

		//-- control handles 
		CDlgCombo m_cmbDataSources;
		
		CDlgCheck m_chkFrame;
		CDlgCheck m_chkFill;
		CDlgCheck m_chkData;
		CDlgCheck m_chkHighlight;
		CDlgCheck m_chkIndicator;
		CDlgCheck m_chkClass;
		CDlgCheck m_chkState;

		CDlgButton m_btnColor[7];	// Frame, Fill, Data, Highlight, Indicator, Class, State
		
		CDlgEdit m_edtChannel;
		
		CDlgCheck m_selWholeFile;
		CDlgCheck m_selList;
		CDlgCheck m_selCurSel;
		
		CDlgButton m_btnLoadData;
		CDlgControl m_picPreview;

		CDlgEdit m_edtMinFrames;
		CDlgEdit m_edtMaxFrames;
		CDlgEdit m_edtSeconds;
		CDlgEdit m_edtImageFile;
		CDlgEdit m_edtSoundFile;
		CDlgButton m_btnPickImage;
		CDlgButton m_btnLoadImages;
		CDlgButton m_btnPickSound;
		CDlgButton m_btnLoadSounds;

		CDlgButton m_btnSave;
		CDlgButton m_btnExpBmp;
		CDlgButton m_btnExpSwf;

		//-- Loaded data			OnBtnLoadData()
		double * m_adData;			// raw data values
		short * m_aiClass;			// raw idealized classes
		short * m_aiState;			// raw idealized states
		int m_iDataAlloc;
		double m_dMin, m_dMax;		// range of loaded data 
		int m_iDataLen;				// # data points ( i.e. dimension of m_adData[] )
		int m_iMaxClass;			// Highest class number in m_aiClass[]
		int m_iMaxState;			// Highest state number in m_aiState[]
		
		//--						// picture frame 
		RECT m_rctPreview;		
		HBRUSH hbrButton;				// for coloring the buttons

		//-- constructor
		CVwMovie() 
			: CDocumentDialog( g_pchMovieFilter, g_pchMovieExtension )
			{ 
			m_iID = IDD_MOVIEVIEW;
			hbrButton = NULL;
			m_adData = NULL;
			m_aiClass = NULL;
			m_aiState = NULL;
			m_iDataAlloc = 0;
			m_iDataPos = 0;
			m_iCapture=NOCAP;	
			m_iXDrag=m_iXHit=m_iYDrag=m_iYHit=0;
			}
			
		//-- destructor
		~CVwMovie() { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			if( m_adData != NULL )
				delete [] m_adData;
			if( m_aiClass != NULL )
				delete [] m_aiClass;
			if( m_aiState != NULL )
				delete [] m_aiState;
			}

		//-- Create arrays to hold data 
		bool AllocateData( int iLen ) { 
			//-- if big enough but not too big ( > 2x ) keep existing buffers.
			if( m_iDataAlloc>=iLen && m_iDataAlloc<iLen*2 ) 
				return true;
			if( m_iDataAlloc>0 ) { 
				delete [] m_adData;
				delete [] m_aiClass;
				delete [] m_aiState;
				}
			if( (m_adData=new double[iLen])==NULL ) 
				return false;
			if( (m_aiClass=new short[iLen])==NULL ) 
				return false;
			if( (m_aiState=new short[iLen])==NULL ) 
				return false;
			m_iDataAlloc=iLen;
			return true;
			}		

		//-- Initialize Controls - set handles, etc.
		void InitializeControls( ) { 
			m_cmbDataSources.SetHandle( GetDlgItem( m_hDlg, IDC_CMB_DATASOURCE ), "Select currently open file as data source for movie" );
		
			m_chkFrame.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_FRAME ), "outline the data display" );
			m_chkFill.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_FILL ), "Fill the data display background" );
			m_chkData.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_DATA ), "Draw the data" );
			m_chkHighlight.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_HIGHLIGHT ), "Change he color of the data as the movie progresses through data" );
			m_chkIndicator.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_INDICATOR ), "Show a vertical frame location indicator in the data" );
			m_chkClass.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_CLASS ), "Draw the idealization class" );
			m_chkState.SetHandle( GetDlgItem( m_hDlg, IDC_CHK_STATE ), "Draw the state" );
			
			m_btnColor[0].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_FRAME ), "Select color for the frame" );
			m_btnColor[1].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_FILL ), "Select color for the background" );
			m_btnColor[2].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_DATA ), "Select color for the data" );
			m_btnColor[3].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_HIGHLIGHT ), "Select color for the data highlighting" );
			m_btnColor[4].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_INDICATORBAR ), "Select color for the indicator" );
			m_btnColor[5].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_CLASS ), "Select color for the idealized class" );
			m_btnColor[6].SetHandle( GetDlgItem( m_hDlg, IDC_BTN_STATE ), "Select color for the idealized state" );
		
			m_edtChannel.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_CHANNEL ), "Data channel to idealize" );

			m_selWholeFile.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_WHOLEFILE ), "" );
			m_selList.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_LIST ), "" );
			m_selCurSel.SetHandle( GetDlgItem( m_hDlg, IDC_SEL_CURSEL ), "" );
		
			m_btnLoadData.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_LOADDATA ), "Copy selected data to movie" );
			m_picPreview.SetHandle( GetDlgItem( m_hDlg, IDC_PIC_PREVIEW ), "Movie frame preview" );
			m_picPreview.GetControlBounds( &m_rctPreview );

			//--
			m_edtMinFrames.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_MINFRAMES ), "Minimum frames per event" );
			m_edtMaxFrames.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_MAXFRAMES ), "Maximum frames per event" );
			m_edtSeconds.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_SECONDS ), "Total time of movie" );
			m_edtImageFile.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_IMAGENAME ), "Name of first image file, should end in '1.bmp'" );
			m_edtSoundFile.SetHandle( GetDlgItem( m_hDlg, IDC_EDT_SOUNDNAME ), "Name of first sound file, should end in '1.wav'" );
			m_btnPickImage.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_PICKIMAGE ), "Open file select dialogue" );
			m_btnLoadImages.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_LOADIMAGES ), "Copy selected images to movie" );
			m_btnPickSound.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_PICKSOUND ), "Open file select dialogue" );
			m_btnLoadSounds.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_LOADSOUNDS ), "Copy selected sounds to movie" );

			m_btnSave.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_SAVE ), "Save movie settings to disk" );
			m_btnExpBmp.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_EXPBMP ), "Export movie as a series of bitmaps" );
			m_btnExpSwf.SetHandle( GetDlgItem( m_hDlg, IDC_BTN_EXPSWF ), "Export movie as a shockwave/flash file" );
			}

		void SetControlValues() { 
			m_selWholeFile.SetCheck( m_oMovie.m_iDataSelect==0 );
			m_selList.SetCheck( m_oMovie.m_iDataSelect==1 );
			m_selCurSel.SetCheck( m_oMovie.m_iDataSelect==2 );
			
			m_edtImageFile.SetText( m_oMovie.m_cImageFile.ConstBuffer() );
			m_edtSoundFile.SetText( m_oMovie.m_cSoundFile.ConstBuffer() );
			FillDataSourceList(0);
			m_cmbDataSources.SetCurSel( m_oMovie.m_cDataName.ConstBuffer() );
			
			m_edtChannel.SetTextInt( m_oMovie.m_iChannel );

			m_chkFrame.SetCheck( m_oMovie.m_bFrame );
			m_chkFill.SetCheck( m_oMovie.m_bFill );
			m_chkData.SetCheck( m_oMovie.m_bData );
			m_chkHighlight.SetCheck( m_oMovie.m_bHighlight );
			m_chkIndicator.SetCheck( m_oMovie.m_bIndicator );
			m_chkClass.SetCheck( m_oMovie.m_bClass );
			m_chkState.SetCheck( m_oMovie.m_bState );
			
			m_edtMinFrames.SetTextInt( m_oMovie.m_iMinFrames );
			m_edtMaxFrames.SetTextInt( m_oMovie.m_iMaxFrames );
			m_edtSeconds.SetTextInt( m_oMovie.m_iMovieSeconds );

			m_btnColor[0].m_cr = m_oMovie.m_crFrame;
			m_btnColor[1].m_cr = m_oMovie.m_crFill;
			m_btnColor[2].m_cr = m_oMovie.m_crData;
			m_btnColor[3].m_cr = m_oMovie.m_crHighlight;
			m_btnColor[4].m_cr = m_oMovie.m_crIndicator;
			m_btnColor[5].m_cr = m_oMovie.m_crClass;
			m_btnColor[6].m_cr = m_oMovie.m_crState;
			}

		void GetControlValues() { 
			m_oMovie.m_iDataSelect = m_selCurSel.GetCheck() ? 2 : m_selList.GetCheck() ? 1 : 0;
			
			m_edtImageFile.GetText( &m_oMovie.m_cImageFile );
			m_edtSoundFile.GetText( &m_oMovie.m_cSoundFile );
			m_cmbDataSources.GetText( &m_oMovie.m_cDataName );
			
			m_oMovie.m_iChannel = m_edtChannel.GetTextInt();

			m_oMovie.m_bFrame = m_chkFrame.GetCheck( );
			m_oMovie.m_bFill = m_chkFill.GetCheck( );
			m_oMovie.m_bData = m_chkData.GetCheck( );
			m_oMovie.m_bHighlight = m_chkHighlight.GetCheck( );
			m_oMovie.m_bIndicator = m_chkIndicator.GetCheck( );
			m_oMovie.m_bClass = m_chkClass.GetCheck( );
			m_oMovie.m_bState = m_chkState.GetCheck( );
			
			m_oMovie.m_iMinFrames = m_edtMinFrames.GetTextInt();
			m_oMovie.m_iMaxFrames = m_edtMaxFrames.GetTextInt( );
			m_oMovie.m_iMovieSeconds = m_edtSeconds.GetTextInt( );

			m_oMovie.m_crFrame = m_btnColor[0].m_cr;
			m_oMovie.m_crFill = m_btnColor[1].m_cr;
			m_oMovie.m_crData = m_btnColor[2].m_cr;
			m_oMovie.m_crHighlight = m_btnColor[3].m_cr;
			m_oMovie.m_crIndicator = m_btnColor[4].m_cr;
			m_oMovie.m_crClass = m_btnColor[5].m_cr;
			m_oMovie.m_crState = m_btnColor[6].m_cr;
			}

		void OnSettingsChanged() { 
			m_bUpdated=true;
			InvalidateRect( m_hDlg, NULL, FALSE );
			//TODO - actions to take if preview is running and settings changed.
			// also invalidate
			}

		void OnSelDataSource() { 
			//todo - if changed, load list names
			}

		void FillDataSourceList( int iSelect ) { 
			m_cmbDataSources.ClearContents();

			//-- add an item for each open data view
			int iSels=0, iWin = WindowListFindID( IDD_DATAVIEW, 0 );
			while( iWin != -1 ) { 
				HWND hwndData = WindowListHandle( iWin );
				char * pchFile = (char *)(((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 1, NULL ));
				m_cmbDataSources.AddString( pchFile );
				++iSels;
				iWin = WindowListFindID( IDD_DATAVIEW, iWin+1 );
				}
				
			if( iSelect>=0 && iSelect<iSels ) 
				m_cmbDataSources.SetCurSel( iSelect );
			}
			
		void OnDropdownDataSource() { 
			//-- remember current selection and clear control contents
			//-- return to selected item, may/may not be same as prior selection
			int iCur = m_cmbDataSources.GetCurSel();
			FillDataSourceList( iCur );
			}
		
		void OnSelDataList() { 
			OnSettingsChanged();		// TODO - neccessary ? 
			}

		//-- For painting the data color buttons
		HBRUSH CreateButtonBrush( int iCtl ) { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			return hbrButton = CreateSolidBrush( m_btnColor[iCtl].m_cr );
			}
			
		void OnBtnColor( int iCtl ) { 
			COLORREF cr = m_btnColor[iCtl].m_cr;
			COLORREF acrTemp[16];
			for( int iCR=0; iCR<16;++iCR ) 
				acrTemp[iCR]=iCR<7 ? m_btnColor[iCR].m_cr : cr ;
				
			CHOOSECOLOR cc;
			ZeroMemory(&cc, sizeof(cc));
			cc.lStructSize = sizeof(cc);
			cc.hwndOwner = m_hDlg;
			cc.lpCustColors = acrTemp; //NULL;	
			cc.rgbResult = cr;
			cc.Flags = CC_FULLOPEN | CC_RGBINIT;	// CC_ANYCOLOR | 
			if( ChooseColor(&cc) ) 
				m_btnColor[iCtl].m_cr=cc.rgbResult;
			InvalidateRect( m_hDlg, NULL, FALSE );

			OnSettingsChanged();
			}

		void OnBtnLoadData() { 
			GetControlValues();

			//-- Obtain ptr to selected file 			
			int iData = m_cmbDataSources.GetCurSel();		// selection # within combo list 
			if( iData < 0 )	{
				MessageBox( GetParent(m_hDlg), "Select a data file to load from.", "Load Data", MB_OK );
				return;
				}
				
			int iWin = WindowListFindID( IDD_DATAVIEW, 0 );
			HWND hwndData;
			while( iWin != -1 ) { 
				hwndData = WindowListHandle( iWin );
				if( m_oMovie.m_cDataName==(char *)(((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 1, NULL )) )
					break;
				iWin = WindowListFindID( IDD_DATAVIEW, iWin+1 );
				}
			if( iWin==-1 ) {
				MessageBox( GetParent(m_hDlg), "Selected data file is unavailable.", "Load Data", MB_OK );
				return;
				}
			CData * pData = (CData *)(((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 2, NULL ));

			//-- Create Selection Set 
			int iChannel=m_oMovie.m_iChannel;
			
			if( iChannel<0 || iChannel>pData->m_oConfig.m_aChannels.Count() ) {		
				MessageBox(NULL, "Channel is out of range.  Loading channel 0.", "GEM Movie", MB_OK );
				iChannel=0;
				}
				
			CSelectionSet oSet;
			bool lSetSetup=false;
			if( m_oMovie.m_iDataSelect==0 ) 
				lSetSetup=oSet.Setup( pData, iChannel );
			else if( m_oMovie.m_iDataSelect==1 ) {
				int iList;
				((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 4, &iList );
				if( iList<0 ) {
					MessageBox( GetParent(m_hDlg), "Selected data file does not have a current list", "Load Data", MB_OK );
					return;
					}
				lSetSetup=oSet.Setup( pData, iChannel, iList );
				}
			else if( m_oMovie.m_iDataSelect==2 ) {
				int ai[4];
				((CDialog *) GetWindowLong( GetParent(hwndData), 0 ))->GetData( 3, ai );
				lSetSetup=oSet.Setup( pData, iChannel, ai[0], ai[1], ai[2], ai[3] );
				}
			
			//-- Process SelectionSet to fill data array
			double d;
			int iPt=0;
			m_iDataLen = oSet.TotalSize();
			bool bInit=false;
			m_iMaxState=0;
			m_iMaxClass=0;
			if( ! AllocateData(m_iDataLen) ) 
				return;
			while( oSet.GetNext() ) {
				bool bIdl = oSet.GetIdealized();
				int iPts=oSet.m_pRange->Size();
				for( int i=0; i<iPts; ++i ) {
					d=oSet.Datum(i);
					m_adData[iPt+i]=d;
					if( bInit ) { 
						if( d<m_dMin ) 
							m_dMin=d;
						if( d>m_dMax ) 
							m_dMax=d;
						}
					else { 
						bInit=true;
						m_dMin=m_dMax=d;
						}
					if( bIdl ) { 
						m_aiClass[iPt+i]=oSet.m_aiClass[i];
						m_aiState[iPt+i]=oSet.m_aiState[i];
						m_iMaxClass = max( m_iMaxClass, m_aiClass[iPt+i] );
						m_iMaxState = max( m_iMaxState, m_aiState[iPt+i] );
						}
					}
				iPt+=oSet.m_pRange->Size();
				}
				
			//-- all done - flag for redraw		
			m_bUpdated=true;
			m_iDataPos = (m_oMovie.m_iX1Data-m_oMovie.m_iX0Data)/2;
			ProcessData();
			}
		
		//-- Convert large dataset to reduced render array 
		void ProcessData() { 
			int iPt=-1;
			int iWid=m_oMovie.m_iX1Data-m_oMovie.m_iX0Data+1;
			int iHgt=m_oMovie.m_iY1Data-m_oMovie.m_iY0Data+1;
			int iHeight;			// offset in disp of data point being processed
			double dSampPerPix=m_iDataLen/iWid;
			double dRange = max(1.0e-6,m_dMax-m_dMin);
			
			//-- allocate render buffers
			if( ! m_oMovie.AllocateRender(iWid) )
				return;
			//-- fill in all 0 if data is empty
			if( dRange==0.0 ) {
				for( iPt=0; iPt<iWid && iPt<m_oMovie.m_iRenderLen; ++iPt ) 
					m_oMovie.m_aiDispLow[iPt]=m_oMovie.m_aiDispHigh[iPt]=0;
				return;
				}
			
			//-- decimate data 
			for( int i=0; i<m_iDataLen; ++i ) { 
				int iCurrPt = min(iWid-1, MulDiv( (i+1), iWid, m_iDataLen )-1 );
				iHeight = max(1, min(iHgt-1, 1+int((m_adData[i]-m_dMin)*(iHgt-2)/dRange) ) );

				while( iPt<iCurrPt /*&& iPt+1<m_oMovie.m_iRenderLen*/ ) { 
					++iPt;
					m_oMovie.m_aiDispClass[iPt]=m_aiClass[i];
					m_oMovie.m_aiDispState[iPt]=m_aiState[i];
					m_oMovie.m_aiDispLow[iPt]=iHeight;
					m_oMovie.m_aiDispHigh[iPt]=iHeight;
					}
				if( iPt>=0 && iPt<m_oMovie.m_iRenderLen ) { 
					m_oMovie.m_aiDispLow[iPt]=min( iHeight, m_oMovie.m_aiDispLow[iPt] );
					m_oMovie.m_aiDispHigh[iPt]=max( iHeight, m_oMovie.m_aiDispHigh[iPt] );
					}
				}

			//-- extend last pt to end of frame for left over (can occur with small selections )
			iPt=max(0,iPt);				
			while( iPt<iWid && iPt<m_oMovie.m_iRenderLen ) {
				m_oMovie.m_aiDispClass[iPt]=m_aiClass[m_iDataLen-1];
				m_oMovie.m_aiDispState[iPt]=m_aiState[m_iDataLen-1];
				m_oMovie.m_aiDispLow[iPt]=iHeight;
				m_oMovie.m_aiDispHigh[iPt]=iHeight;
				++iPt;
				}
				
			//-- Cause Redraw
			InvalidateRect( m_hDlg, NULL, FALSE );
			}

		//--- painting (somewhat less than) primitives 
		void DrawLineBox( HDC hdc, int iX0, int iY0, int iX1, int iY1 ) { 
			MoveToEx( hdc, iX0, iY0, NULL );
			LineTo( hdc, iX0, iY1 );
			LineTo( hdc, iX1, iY1 );
			LineTo( hdc, iX1, iY0 );
			LineTo( hdc, iX0, iY0 );
			}

		void DrawFillBox( HDC hdc, int iX0, int iY0, int iX1, int iY1, COLORREF cr ) { 
			//-- Draw frame and fill 
			HBRUSH brWas, brFill = CreateSolidBrush(cr);
			brWas = (HBRUSH) SelectObject( hdc, brFill );
			Rectangle( hdc, iX0, iY0, iX1, iY1 );
			SelectObject( hdc, brWas );
			DeleteObject( brFill );
			}

		void DrawMouseBlocks( HDC hdc, int iX0, int iY0, int iX1, int iY1, COLORREF cr ) { 
			//-- Mouse draggable blocks 
			RECT rctBlock;
			HBRUSH brBlock = CreateSolidBrush(cr);
			SetRect( &rctBlock, iX0, iY0, iX0+DRAGBLOCKSIZE, iY0+DRAGBLOCKSIZE );
			FillRect( hdc, &rctBlock, brBlock );
			SetRect( &rctBlock, iX1-DRAGBLOCKSIZE, iY1-DRAGBLOCKSIZE, iX1, iY1 );
			FillRect( hdc, &rctBlock, brBlock );
			DeleteObject( brBlock );
			}

		void DrawIndicator( HDC hdc, int iX, int iY0, int iY1, COLORREF cr ) { 
			HPEN penDraw = CreatePen( PS_SOLID, 0, cr );
			HPEN penWas = (HPEN) SelectObject( hdc, penDraw );
			MoveToEx( hdc, iX, iY0+1, NULL );
			LineTo( hdc, iX, iY1-1 );
			SelectObject( hdc, penWas );
			DeleteObject( penDraw );
			}

		//--				
		void OnPaint() {
			/*
			//--- Don't continuously redraw while sizing
			if( m_bSizing ) 
				return;
			*/

			//------------------------------------------------------ Update the display
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(m_hDlg, &ps);
			HPEN penWas, penDraw, penHighlight;

			if( m_oMovie.m_aBMP.Count()>0 ) { 
				int iState = m_iDataPos<0 || m_iDataPos>=m_oMovie.m_iRenderLen ? 0 : m_oMovie.m_aiDispState[m_iDataPos];
				if( iState<0 || iState>m_oMovie.m_aBMP.Count() )
					iState=0;
					
				CBMP * pBmp = m_oMovie.BMP(iState);
				
				//-- Create bitmap for buffering display 
				int iWid = m_rctPreview.right-m_rctPreview.left-1;
				int iHgt = m_rctPreview.bottom-m_rctPreview.top-1;
				HDC hdcBitmap = CreateCompatibleDC( hdc );
				HBITMAP hBitmap = CreateCompatibleBitmap( hdc, iWid, iHgt );
				SelectObject( hdcBitmap, hBitmap );

				//-- copy loaded bitmap data to bitmap device context 
				// TODO - GetDIBits	for this ? 
				BYTE * pSrc;
				for( int iRow=0; iRow<iHgt && iRow<pBmp->m_iHgt; ++iRow ) { 
					for( int iCol=0; iCol<iWid && iCol<pBmp->m_iWid; ++iCol ) { 
						pSrc = pBmp->Pixel(iRow,iCol);
						SetPixel( hdcBitmap, iCol, iHgt-iRow, RGB(pSrc[2], pSrc[1], pSrc[0]) );
						}
					}
					
				//-- copy device context bitmap to display 
				if( ! BitBlt( hdc, 
						m_rctPreview.left+1, 
						m_rctPreview.top+1, 
						m_rctPreview.right-m_rctPreview.left-1, 
						m_rctPreview.bottom-m_rctPreview.top-1,	 
						hdcBitmap,							// Source bitmap 
						0,										// Source Left
						0,										// Source Top 
						SRCCOPY ) ) {
					DWORD dwGLE = GetLastError();
					++dwGLE;
					}
						
				DeleteObject( hBitmap );
				DeleteDC( hdcBitmap );
				}

			//------------------------------------------ Data box 
			if( m_iCapture==DATAMOVE || m_iCapture==DATASIZE ) { 
				HPEN penDragFrame = CreatePen( PS_SOLID, 0, m_oMovie.m_crFrame );
				HPEN penWas = (HPEN) SelectObject( hdc, penDragFrame );
				int iDX=(m_iXDrag-m_iXHit);
				int iDY=(m_iYDrag-m_iYHit);
				if( m_iCapture==DATAMOVE ) {		// moving data box
					int iX=max( 0, iDX + m_oMovie.m_iX0Data );
					int iY=max( 0, iDY + m_oMovie.m_iY0Data );
					DrawLineBox( hdc, m_rctPreview.left+iX, m_rctPreview.top+iY, 
							 m_rctPreview.left+iX+(m_oMovie.m_iX1Data-m_oMovie.m_iX0Data),
							 m_rctPreview.top+iY+(m_oMovie.m_iY1Data-m_oMovie.m_iY0Data) );
					}
				else if( m_iCapture==DATASIZE ) {		// sizing data box
					int iX=max( m_oMovie.m_iX0Data+16, iDX + m_oMovie.m_iX1Data );
					int iY=max( m_oMovie.m_iY0Data+16, iDY + m_oMovie.m_iY1Data );
					DrawLineBox( hdc, m_rctPreview.left+m_oMovie.m_iX0Data, 
							  m_rctPreview.top+m_oMovie.m_iY0Data, 
							  m_rctPreview.left+iX,
							 m_rctPreview.top+iY);
					}
				SelectObject( hdc, penWas );
				DeleteObject( penDragFrame );
				}
			else if( m_oMovie.m_iRenderLen>0 ) { 
				int iX0 = m_rctPreview.left+m_oMovie.m_iX0Data;
				int iY0 = m_rctPreview.top+m_oMovie.m_iY0Data;
				int iX1 = m_rctPreview.left+m_oMovie.m_iX1Data;
				int iY1 = m_rctPreview.top+m_oMovie.m_iY1Data;
				int iWid = iX1-iX0+1;

				// Data Frame and fill 
				if( m_oMovie.m_bFill || m_oMovie.m_bFrame ) { 
					penDraw = CreatePen( PS_SOLID, 0, m_oMovie.m_bFrame ? m_oMovie.m_crFrame : m_oMovie.m_crFill );
					penWas = (HPEN) SelectObject( hdc, penDraw );
					if( m_oMovie.m_bFill )
						DrawFillBox( hdc, iX0, iY0, iX1, iY1, m_oMovie.m_crFill );
					else
						DrawLineBox( hdc, iX0, iY0, iX1, iY1 );
					SelectObject( hdc, penWas );
					DeleteObject( penDraw );
					}
				
				//-- Data 
				if( m_oMovie.m_bData ) { 
					penHighlight = CreatePen( PS_SOLID, 0, m_oMovie.m_crHighlight );
					penDraw = CreatePen( PS_SOLID, 0, m_oMovie.m_crData );
					penWas = (HPEN) SelectObject( hdc, m_iDataPos>0 && m_oMovie.m_bHighlight ? penHighlight : penDraw );
					int iLow, iHigh;
					for( int iPt=0; iPt<iWid; ++iPt ) { 
						//-- switch pens when past highlight pt.
						if( iPt==m_iDataPos )
							SelectObject( hdc, penDraw );
						//-- low/ high pixel at current pt
						iLow = m_oMovie.m_aiDispLow[iPt];
						iHigh = m_oMovie.m_aiDispHigh[iPt];
						//-- connect from prior pt to this one
						if( iPt > 0 ) {
							if( m_oMovie.m_aiDispHigh[iPt-1]<iLow ) 
								iLow = m_oMovie.m_aiDispHigh[iPt-1];
							else if( m_oMovie.m_aiDispLow[iPt-1]>iHigh ) 
								iHigh = m_oMovie.m_aiDispLow[iPt-1];
							}
						//-- set pixel or draw vertical line for this pt
						if( iLow==iHigh ) 
							SetPixel( hdc, iX0+iPt, iY1-iLow, 
									iPt>=m_iDataPos || !m_oMovie.m_bHighlight ? m_oMovie.m_crData : m_oMovie.m_crHighlight );
						else {
							MoveToEx( hdc, iX0+iPt, iY1-iLow, NULL );
							LineTo( hdc, iX0+iPt, iY1-iHigh );
							}
						}
					SelectObject( hdc, penWas );
					DeleteObject( penHighlight );
					DeleteObject( penDraw );
					}
					
				if( m_oMovie.m_bIndicator )
					DrawIndicator( hdc, iX0+m_iDataPos, iY0, iY1, m_oMovie.m_crIndicator );
					
				//-- TODO Idealized Class
				DrawMouseBlocks( hdc, iX0, iY0, iX1, iY1, m_oMovie.m_crFrame );
				}

			//------------------------------------------ State box 
			if( m_iCapture==STATEMOVE || m_iCapture==STATESIZE ) { 
				HPEN penDragFrame = CreatePen( PS_SOLID, 0, m_oMovie.m_crFrame );
				HPEN penWas = (HPEN) SelectObject( hdc, penDragFrame );
				int iDX=(m_iXDrag-m_iXHit);
				int iDY=(m_iYDrag-m_iYHit);
				if( m_iCapture==STATEMOVE ) {		// move state display
					int iX=max( 0, iDX + m_oMovie.m_iX0State );
					int iY=max( 0, iDY + m_oMovie.m_iY0State );
					DrawLineBox( hdc, m_rctPreview.left+iX, m_rctPreview.top+iY, 
							 m_rctPreview.left+iX+(m_oMovie.m_iX1State-m_oMovie.m_iX0State),
							 m_rctPreview.top+iY+(m_oMovie.m_iY1State-m_oMovie.m_iY0State) );
					}
				else if( m_iCapture==STATESIZE ) {		// sizing state display
					int iX=max( m_oMovie.m_iX0State+16, iDX + m_oMovie.m_iX1State );
					int iY=max( m_oMovie.m_iY0State+16, iDY + m_oMovie.m_iY1State );
					DrawLineBox( hdc, m_rctPreview.left+m_oMovie.m_iX0State, 
							  m_rctPreview.top+m_oMovie.m_iY0State, 
							  m_rctPreview.left+iX,
							 m_rctPreview.top+iY);
					}
				SelectObject( hdc, penWas );
				DeleteObject( penDragFrame );
				}
			else if( m_oMovie.m_bState && m_oMovie.m_iRenderLen>0 ) {
				//---------- Draw state box 
				//-- Idealized State 
				if( m_oMovie.m_bState ) { 
					int iX0 = m_rctPreview.left+m_oMovie.m_iX0State;
					int iY0 = m_rctPreview.top+m_oMovie.m_iY0State;
					int iX1 = m_rctPreview.left+m_oMovie.m_iX1State;
					int iY1 = m_rctPreview.top+m_oMovie.m_iY1State;
					int iWid = iX1-iX0+1;

					//-- State Frame and fill 
					if( m_oMovie.m_bFill || m_oMovie.m_bFrame ) { 
						penDraw = CreatePen( PS_SOLID, 0, m_oMovie.m_bFrame ? m_oMovie.m_crFrame : m_oMovie.m_crFill );
						penWas = (HPEN) SelectObject( hdc, penDraw );
						if( m_oMovie.m_bFill )
							DrawFillBox( hdc, iX0, iY0, iX1, iY1, m_oMovie.m_crFill );
						else
							DrawLineBox( hdc, iX0, iY0, iX1, iY1 );
						SelectObject( hdc, penWas );
						DeleteObject( penDraw );
						}
				
					for( int iPt=0; iPt<iWid && iPt<m_oMovie.m_iRenderLen; ++iPt ) { 
						int iSt = m_oMovie.m_aiDispState[iPt];
						if( iSt>=0 && iSt<12 )
							SetPixel( hdc, iX0+iPt, iY1-10*iSt, m_oMovie.m_crState );
						}

					if( m_oMovie.m_bIndicator )
						DrawIndicator( hdc, iX0+m_iDataPos, iY0, iY1, m_oMovie.m_crIndicator );
					DrawMouseBlocks( hdc, iX0, iY0, iX1, iY1, m_oMovie.m_crFrame );
					}
				}
		
			EndPaint(m_hDlg, &ps);
			}

		void OnBtnPickImage() { 
			//-- select the file to load 
			char achLongName[_MAX_PATH]={0};
			char achFileName[_MAX_PATH]={0};
			m_edtImageFile.GetText( achLongName, _MAX_PATH );
			if( ! GetOpenFileName( achLongName, 
							"Bmp files (*.bmp)\0*.bmp\0All Files (*.*)\0*.*\0\0", 
							"bmp", GetParent(m_hDlg), achFileName ) ) 
				return;
			m_edtImageFile.SetText( achLongName );
			}

		void OnBtnPickSound() { 
			//-- select the file to load 
			char achLongName[_MAX_PATH]={0};
			char achFileName[_MAX_PATH]={0};
			m_edtSoundFile.GetText( achLongName, _MAX_PATH );
			if( ! GetOpenFileName( achLongName, 
							"Sound files (*.wav)\0*.wav\0All Files (*.*)\0*.*\0\0", 
							"wav", GetParent(m_hDlg), achFileName ) ) 
				return;
			m_edtSoundFile.SetText( achLongName );
			}

		void OnBtnLoadImages() { 
			CString cFile;
			m_edtImageFile.GetText( &cFile );
			if( cFile.Length()==0 ) 
				OnBtnPickImage();
			m_edtImageFile.GetText( &cFile );
			if( cFile.Length()==0 ) 
				return;
			m_oMovie.LoadBitmaps( cFile.ConstBuffer(), -1 );

			char bufMsg[100];
			sprintf( bufMsg, "%d image files loaded.", m_oMovie.m_aBMP.Count() );
			SetWindowText( GetDlgItem( m_hDlg, IDC_LBL_IMAGEFILES ), bufMsg );
			
			InvalidateRect( m_hDlg, NULL, FALSE );
			ResizePreview();
			}

		void OnBtnLoadSounds() { 
			CString cFile;
			m_edtSoundFile.GetText( &cFile );
			if( cFile.Length()==0 ) 
				OnBtnPickSound();
			m_edtSoundFile.GetText( &cFile );
			if( cFile.Length()==0 ) 
				return;

			m_oMovie.LoadSounds( cFile.ConstBuffer(), -1 );

			char bufMsg[100];
			sprintf( bufMsg, "%d sound files loaded.", m_oMovie.m_aWAV.Count() );
			SetWindowText( GetDlgItem( m_hDlg, IDC_LBL_SOUNDFILES ), bufMsg );

			}

		void OnBtnExportBMP() { 
			GetControlValues();
			//-- select the file to load 
			char achLongName[_MAX_PATH]={0};
			char achFileName[_MAX_PATH]={0};
			strcpy( achLongName, m_oMovie.m_cBMPName.ConstBuffer() );
			if( ! GetSaveFileName( achLongName, 
							"BMP files (*.bmp)\0*.bmp\0All Files (*.*)\0*.*\0\0", 
							"bmp", GetParent(m_hDlg), achFileName ) ) 
				return;

			if( m_oMovie.ExportBmp( achLongName, m_iDataLen, m_aiState ) ) 
				MessageBox( NULL, "Completed", "Export BMP", MB_OK );
			}

		void OnBtnExportSWF() { 
			//-- select the file to load 
			char achLongName[_MAX_PATH]={0};
			char achFileName[_MAX_PATH]={0};
			strcpy( achLongName, m_oMovie.m_cSWFName.ConstBuffer() );
			if( ! GetSaveFileName( achLongName, 
							"Shockwave files (*.swf)\0*.swf\0All Files (*.*)\0*.*\0\0", 
							"swf", GetParent(m_hDlg), achFileName ) ) 
				return;

			m_oMovie.ExportSwf( achLongName, m_iDataLen, m_aiState );
			}

		void OnClickData() { 
			MessageBox( NULL, "Under Construction", "GEM Movie", MB_OK );
			// TODO User clicked in data area to change the frame 
			}
			
		void OnFrameAdvance(){ 
			// TODO respond to timer message to advance to next frame 
			}

		//-----------------------------------------------------
		//-- Sent when dialogue exit occurs 
		void OnExit() { 
			//TODO - terminate timer if running 
//			if( m_bUpdated ) 
//				OnSave();
			}

		//---- Create a new data object 
		void OnNewMovie(  ) { 
			SetControlValues();
			//TODO 
			}

		//-- Check if user wants to save file before closing window.  false if user cancels save.		
		bool Close( ) { 
			return ! Updated() || OnSave(); 
			}	

		//----- Check if a window can be closed.
		bool CanClose( ) { 
			return m_oMovie.GetStatus()==0; 
			}	

		//---- Open an existing movie file
		void OnLoadFile( const char * pchFileName ) { 
			if( m_oMovie.LoadFile( pchFileName ) ) { 
				SetWindowText( GetParent(m_hDlg), pchFileName );
				m_cFileName.Set( pchFileName );
				SetControlValues();
				}
			else
				MessageBox( GetParent(m_hDlg), "Error opening movie file", pchFileName, MB_OK );
			OnSettingsChanged();
			}

		void ResizePreview() { 
			//-- dialogue window info 
			m_picPreview.GetControlBounds( &m_rctPreview );
			m_rctPreview.left--;
			m_rctPreview.top--;
			m_rctPreview.right = m_rctPreview.left + m_oMovie.m_iBMPMaxWidth+1;
			m_rctPreview.bottom = m_rctPreview.top + m_oMovie.m_iBMPMaxHeight+1;
			MoveWindow( m_picPreview.GetHandle(), m_rctPreview.left, m_rctPreview.top, 
						m_oMovie.m_iBMPMaxWidth+2, 
						m_oMovie.m_iBMPMaxHeight+2, 
						TRUE );
			InvalidateRect( m_hDlg, NULL, FALSE );
			}
	
		void OnSize( WPARAM wParam, LPARAM * p ) { 
			// Window resizing, including the initial sizing 

			//-- Desired size
			//int iDlgWid=LOWORD(*p);
			//int iDlgHgt=HIWORD(*p);

			ResizePreview();
			}

		//---- 
		bool OnSave() { 
			if( ! GetSaveAsFileName( ) )
				return false;
			GetControlValues();
			m_oMovie.SaveFile( m_cFileName.ConstBuffer() );
			UpdateMovieMRU( m_cFileName.ConstBuffer() );
			return true;
			}

		//--
		bool MouseDown( short iX, short iY ) { 
			int iXX = iX-m_rctPreview.left;
			int iYY = iY-m_rctPreview.top;
			m_iCapture=NOCAP;
			//-- Click in data display 
			if( iXX>=m_oMovie.m_iX0Data && iXX<=m_oMovie.m_iX0Data+DRAGBLOCKSIZE 
					&& iYY>=m_oMovie.m_iY0Data && iYY<=m_oMovie.m_iY0Data+DRAGBLOCKSIZE )
				m_iCapture=DATAMOVE;
			else if( m_oMovie.m_bState 
					&& iXX>=m_oMovie.m_iX0State && iXX<=m_oMovie.m_iX0State+DRAGBLOCKSIZE 
					&& iYY>=m_oMovie.m_iY0State && iYY<=m_oMovie.m_iY0State+DRAGBLOCKSIZE )
				m_iCapture=STATEMOVE;
			else if( iXX>=m_oMovie.m_iX1Data-DRAGBLOCKSIZE && iXX<=m_oMovie.m_iX1Data
					&& iYY>=m_oMovie.m_iY1Data-DRAGBLOCKSIZE && iYY<=m_oMovie.m_iY1Data )
				m_iCapture=DATASIZE;
			else if( m_oMovie.m_bState 
					&& iXX>=m_oMovie.m_iX1State-DRAGBLOCKSIZE && iXX<=m_oMovie.m_iX1State
					&& iYY>=m_oMovie.m_iY1State-DRAGBLOCKSIZE && iYY<=m_oMovie.m_iY1State )
				m_iCapture=STATESIZE;
			else if( iXX>=m_oMovie.m_iX0Data && iXX<=m_oMovie.m_iX1Data
					&& iYY>=m_oMovie.m_iY0Data && iYY<=m_oMovie.m_iY1Data ) { 
				//-- click inside data box - change frame (data highlight position)
				m_iCapture=DATAFRAME;
				m_iDataPos = iXX-m_oMovie.m_iX0Data;
				InvalidateRect( m_hDlg, NULL, FALSE );
				}
			else if( m_oMovie.m_bState 
					&& iXX>=m_oMovie.m_iX0State && iXX<=m_oMovie.m_iX1State
					&& iYY>=m_oMovie.m_iY0State && iYY<=m_oMovie.m_iY1State ) { 
				//-- click inside state box - change frame
				m_iCapture=STATEFRAME;
				m_iDataPos = iXX-m_oMovie.m_iX0State;
				InvalidateRect( m_hDlg, NULL, FALSE );
				}

			//-- if anything good was hit, remember where it was 					
			if( m_iCapture!=NOCAP ) {
				SetCapture( m_hDlg );
				m_iXDrag=m_iXHit=iX;
				m_iYDrag=m_iYHit=iY;
				return TRUE;
				}

			return true;	
			}

		bool MouseMove( short iX, short iY ) { 
			if( m_iCapture==DATAFRAME || m_iCapture==STATEFRAME ) { 
				//-- Click in data - change highlight position 
				m_iDataPos = iX - m_rctPreview.left - ( m_iCapture==STATEFRAME ? m_oMovie.m_iX0State :m_oMovie.m_iX0Data );
				m_iDataPos = min( ( m_iCapture==STATEFRAME ? m_oMovie.m_iX1State-m_oMovie.m_iX0State : m_oMovie.m_iX1Data-m_oMovie.m_iX0Data )+1, max( 0, m_iDataPos ) );
				}

			m_iXDrag=iX;
			m_iYDrag=iY;
			InvalidateRect( m_hDlg, NULL, FALSE );
			return true;
			}
			
		bool MouseUp( short iX, short iY ) { 
			ReleaseCapture();
			if( m_iCapture==DATAFRAME || m_iCapture==STATEFRAME ) { 
				m_iDataPos = iX-m_rctPreview.left-( m_iCapture==STATEFRAME ? m_oMovie.m_iX0State : m_oMovie.m_iX0Data) ;
				m_iDataPos = min( ( m_iCapture==STATEFRAME ? m_oMovie.m_iX1State-m_oMovie.m_iX0State : m_oMovie.m_iX1Data-m_oMovie.m_iX0Data)+1, max( 0, m_iDataPos ) );
				}
			else if( m_iCapture==DATAMOVE ) { 
				//-- adjust data location
				int iWid=m_oMovie.m_iX1Data-m_oMovie.m_iX0Data;		
				int iHgt=m_oMovie.m_iY1Data-m_oMovie.m_iY0Data;
				m_oMovie.m_iX0Data = max( m_oMovie.m_iX0Data + (m_iXDrag-m_iXHit), 0 );
				m_oMovie.m_iY0Data = max( m_oMovie.m_iY0Data + (m_iYDrag-m_iYHit), 0 );
				m_oMovie.m_iX1Data = m_oMovie.m_iX0Data+iWid;
				m_oMovie.m_iY1Data = m_oMovie.m_iY0Data+iHgt;
				}
			else if( m_iCapture==STATEMOVE ) { 
				//-- adjust data location
				int iWid=m_oMovie.m_iX1State-m_oMovie.m_iX0State;		
				int iHgt=m_oMovie.m_iY1State-m_oMovie.m_iY0State;
				m_oMovie.m_iX0State = max( m_oMovie.m_iX0State + (m_iXDrag-m_iXHit), 0 );
				m_oMovie.m_iY0State = max( m_oMovie.m_iY0State + (m_iYDrag-m_iYHit), 0 );
				m_oMovie.m_iX1State = m_oMovie.m_iX0State+iWid;
				m_oMovie.m_iY1State = m_oMovie.m_iY0State+iHgt;
				}
			else if( m_iCapture==DATASIZE ) {
				//-- adjust data size and state width 
				m_oMovie.m_iX1Data = max(m_oMovie.m_iX0Data+16,m_oMovie.m_iX1Data+(m_iXDrag-m_iXHit));
				m_oMovie.m_iY1Data = max(m_oMovie.m_iY0Data+16,m_oMovie.m_iY1Data+(m_iYDrag-m_iYHit));
				m_oMovie.m_iX1State = m_oMovie.m_iX0State + ( m_oMovie.m_iX1Data - m_oMovie.m_iX0Data );
				}
			else if( m_iCapture==STATESIZE ) {
				//-- adjust State size and data width 
				m_oMovie.m_iX1State = max(m_oMovie.m_iX0State+16,m_oMovie.m_iX1State+(m_iXDrag-m_iXHit));
				m_oMovie.m_iY1State = max(m_oMovie.m_iY0State+16,m_oMovie.m_iY1State+(m_iYDrag-m_iYHit));
				m_oMovie.m_iX1Data = m_oMovie.m_iX0Data + ( m_oMovie.m_iX1State - m_oMovie.m_iX0State );
				}
			m_iCapture=NOCAP;
			ProcessData();		// re-render and invalidate 
			return true;
			}

		void * GetData( int iData, void * pInfo ) { 
			if( iData == 1 ) 
				return m_cFileName.Buffer();
			return NULL;
			}
		};


//-----------------------------------------------------------------------------
BOOL CALLBACK MovieViewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	CVwMovie * pVw;
	
	//-- create the CVwMovie to go with the dialog 
	if( iMsg == WM_INITDIALOG ) { 
		pVw = new CVwMovie;
		pVw->m_hDlg=hDlg;
		pVw->InitializeControls();
		pVw->OnMDIActivate( NULL, GetParent(hDlg) );
		SetWindowLong( GetParent(hDlg), 0, (long) pVw ) ;
		return TRUE;
		}
	
	// Get the CVwData pointer for this dialog 
	pVw = (CVwMovie *) GetWindowLong(  GetParent( hDlg ), 0 );

	int wmId, wmEvent;

	switch( iMsg ) { 
		//-----
		case WM_LBUTTONDOWN :		//-- Start of drag operation			
			return pVw->MouseDown( LOWORD(lParam), HIWORD(lParam) ) ? TRUE : FALSE;

		case WM_MOUSEMOVE :			//-- Mouse move during drag operation
			if( pVw->m_iCapture == NOCAP ) 
				return FALSE;
			pVw->MouseMove( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;

		case WM_LBUTTONUP :			//-- end of drag operation
			if( pVw->m_iCapture == NOCAP ) 
				return FALSE;
			pVw->MouseUp( LOWORD(lParam), HIWORD(lParam) );
			return TRUE;

		case WM_MOVIE_NEWFILE :
			pVw->OnNewMovie();
			break;

		case WM_MOVIE_LOADFILE :
			pVw->OnLoadFile( (char *) lParam );
			break;

		case WM_CTLCOLORBTN :
			wmId = GetDlgCtrlID( (HWND)lParam );
			if( wmId==IDC_BTN_FRAME )
				return (BOOL) pVw->CreateButtonBrush(0);
			else if( wmId==IDC_BTN_FILL )
				return (BOOL) pVw->CreateButtonBrush(1);
			else if( wmId==IDC_BTN_DATA )
				return (BOOL) pVw->CreateButtonBrush(2);
			else if( wmId==IDC_BTN_HIGHLIGHT )
				return (BOOL) pVw->CreateButtonBrush(3);
			else if( wmId==IDC_BTN_INDICATORBAR )
				return (BOOL) pVw->CreateButtonBrush(4);
			else if( wmId==IDC_BTN_CLASS )
				return (BOOL) pVw->CreateButtonBrush(5);
			else if( wmId==IDC_BTN_STATE )
				return (BOOL) pVw->CreateButtonBrush(6);
			return FALSE;

		case WM_PAINT :
			pVw->OnPaint();
			return FALSE;

		case WM_COMMAND :
			wmId = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch( wmId ) { 
				//-- Selection List message handlers
				case IDC_CMB_DATASOURCE :
					if( wmEvent==CBN_DROPDOWN )
						pVw->OnDropdownDataSource();
					if( wmEvent==CBN_SELCHANGE )
						pVw->OnSelDataSource();
					break;

				case IDC_CMB_LISTS :
					if( wmEvent==CBN_SELCHANGE )
						pVw->OnSelDataList();
					break;
					
				case IDC_BTN_LOADDATA:
					pVw->OnBtnLoadData();
					break;

				case IDC_BTN_PICKIMAGE:
					pVw->OnBtnPickImage();
					break;
				case IDC_BTN_LOADIMAGES:
					pVw->OnBtnLoadImages();
					break;
				case IDC_BTN_PICKSOUND:
					pVw->OnBtnPickSound();
					break;
				case IDC_BTN_LOADSOUNDS:
					pVw->OnBtnLoadSounds();
					break;

				case IDC_BTN_SAVE:
					pVw->OnSave();
					break;
				case IDC_BTN_EXPBMP:
					pVw->OnBtnExportBMP();
					break;
				case IDC_BTN_EXPSWF:
					pVw->OnBtnExportSWF();
					break;

				case IDC_BTN_FRAME:
					pVw->OnBtnColor( 0 );
					break;
				case IDC_BTN_FILL:
					pVw->OnBtnColor( 1 );
					break;
				case IDC_BTN_DATA:
					pVw->OnBtnColor( 2 );
					break;
				case IDC_BTN_HIGHLIGHT:
					pVw->OnBtnColor( 3 );
					break;
				case IDC_BTN_INDICATORBAR:
					pVw->OnBtnColor( 4 );
					break;
				case IDC_BTN_CLASS:
					pVw->OnBtnColor( 5 );
					break;
				case IDC_BTN_STATE:
					pVw->OnBtnColor( 6 );
					break;

				default : 
					return FALSE;
				}
			return TRUE;

		case WM_DESTROY :
			pVw->OnExit();	
			delete pVw;
			SetWindowLong( GetParent(hDlg), 0, 0 ) ;
			WindowListRemove( hDlg );
			return TRUE;
		
		default : 
			break;
		}
		
	return FALSE;
	} 

